<?php
/**
 * Gentics Portal.Node PHP
 * Author & Copyright (c) by Gentics Software GmbH
 * sales@gentics.com
 * http://www.gentics.com
 * Licenses can be found in the LICENSE.txt file in the root-folder of this installation
 * You must not use this software without a valid license agreement.
 *
 * Pdf module class file.
 */
class PdfModule extends CWebModule
{
    /**
     * @var string a secret salt for generating pdf from url, use random string
     */
    public $salt = '';
    
    /**
     * @var string temporary path for created pdf files, if not set system tmp dir is used
     */
    public $pdfWebPath = '';
    
    /**
     * @var string path to wkhtmltopdf binary
     */
    public $wkhtmltopdfBin = '/usr/local/bin/wkhtmltox/bin/wkhtmltopdf';
    /**
     * @var array wkhtmltopdf options for all pdf files, set options in 
     * key/value pairs. for available options see http://wkhtmltopdf.org/usage/wkhtmltopdf.txt
     */
    public $globalOptions = array();
    
    /**
     * @var string, only use when a ContentRepository is available and ?pdf=true is used. 
     * If set a ContentRepository Request for the actual page is done to fetch the given attribut. 
     * The ContentRepository Value should contain a json encoded string with custom paramters. 
     * This paramters will additionally be used as wkhtmltopdf options.
     */
    public $optionsParamterCCR = null;
    
    /**
     * @var boolean, when true Content-Disposition in Header will be set to inline
     */
    public $inline = true;
    
    /**
     * @var boolean, when set X-Robots-Tag:noindex will be set in Header
     */
    public $robotsNoIndex = false;
    
    /**
     * @var boolean $createByParam allow pdf creation by adding pdf=true as url paramter
     */
    public $createByParam = false;
    
    /**
     * init - import pdf libs and check if a salt is set.
     *
     * @return void
     */
    public function init()
    {
        parent::init();
        $this->setImport(array(
            'pdf.phpwkhtmltopdf-master.src.*'
        ));
        if ($this->salt == null) {
            throw new Exception("please set a secret salt for pdf generation in your config");
        }
    }
    
    /**
     * generate a md5 hash for security paramter in generation url
     *
     * @param string $string this string get hashed
     */
    public function hash($string)
    {
        if ($this->salt == null) {
            throw new Exception("please set a secret salt for pdf generation in your config");
            return;
        }
        return md5(md5($string . $this->salt) . $this->salt);
    }
    
    /**
     * create a url for a given string and optional pdf options (wkhtmltopdf options)
     * 
     * @param string $pageUrl a pdf is generated for this url if a client request the created url
     * @param string $options a json encoded string containing an array (key => value) 
     * with wkhtmltopdf options
     * 
     * @return string url
     */
    public function createUrl($pageUrl, $options = null)
    {
        if ($options != null) {
            return Yii::app()->createAbsoluteUrl('/pdf/gen', array(
                'url' => $pageUrl,
                'key' => $this->hash($pageUrl . $options),
                'pdfoptions' => $options
            ));
        } else {
            return Yii::app()->createAbsoluteUrl('/pdf/gen', array(
                'url' => $pageUrl,
                'key' => $this->hash($pageUrl)
            ));
        }
    }
    
    /**
     * Create a pdf from given url. If no url isset, create pdf from User Request
     * Never call this method with $pdfoptiosn set by user-Input!. Only via createUrl Method.
     *
     * @param string $request pdf of this url get created
     * @param string $pdfoptions json encoded string with options for wkhtmltopdf
     */
    public function createPdf($request = null, $pdfoptions = null)
    {
        //if no url is set, use actual url and strip pdf=true from url.
        if ($request == null) {
            //set request url, strip pdf=true
            $request = Yii::app()->request->hostInfo . Yii::app()->request->url;
            $request = str_replace("pdf=true", "", $request);
        }
        
        $pdf = new Pdf($request);
        if (isset($this->pdfWebPath) && $this->pdfWebPath != '') {
            $pdf->tmpDir = $this->pdfWebPath;
        }
        $pdf->binary         = $this->wkhtmltopdfBin;
        $pdf->ignoreWarnings = true;
        
        //set global options
        $options = array();
        if (is_array($this->globalOptions) && sizeof($this->globalOptions) > 0) {
            $options = $this->globalOptions;
        }
        
        /*
        check if tagmap entry with options is set. 
        options should be written to CCR in json (e.g. {"title":"neuerTitel","test":"testtext"})
        */
        if (isset($this->optionsParamterCCR) && $this->optionsParamterCCR != "") {
            $cmsObject = Yii::app()->getComponent('repositoryApi')->getCmsObject("/" . Yii::app()->request->getParam('path'), array(
                $this->optionsParamterCCR
            ));
            if (isset($cmsObject) && isset($cmsObject['attributes']) && isset($cmsObject['attributes'][$this->optionsParamterCCR])
                && $cmsObject['attributes'][$this->optionsParamterCCR] != ""
            ) {
                $json  = $cmsObject['attributes'][$this->optionsParamterCCR];
                $array = json_decode($json, true);
                if (is_array($array)) {
                    $options = array_merge($options, $array);
                }
            }
        }
        //add pdfoptions to options-array
        if ($pdfoptions != null) {
            $array = json_decode($pdfoptions, true);
            if (is_array($array)) {
                $options = array_merge($options, $array);
            }
        }
        
        //strip title from options and use as pdf filename
        if (is_array($options) && sizeof($options) > 0) {
            if (isset($options['title'])) {
                $title = $options['title'];
                unset($options['title']);
            }
            $pdf->setOptions($options);
        }
        
        
        //generate and send PDF to Browser
        if (isset($title) && $title != "") {
            $pdf->send($title, $this->inline, $this->robotsNoIndex);
        } else {
            $pdf->send(null, $this->inline, $this->robotsNoIndex);
        }
    }    
}
