<?php
/**
 * Gentics Portal.Node PHP
 * Author & Copyright (c) by Gentics Software GmbH
 * sales@gentics.com
 * http://www.gentics.com
 * Licenses can be found in the LICENSE.txt file in the root-folder of this installation
 * You must not use this software without a valid license agreement.
 *
 */
class SearchApi extends CComponent
{
    /**
     * Overrided search url
     *
     * @public string
     **/
    public $searchUrl;

    /**
     * Overrided autosuggest url
     *
     * @public string
     **/
    public $autosuggestUrl;

    private $_repositoryApi;

    /**
     * Internal getter for repository api.
     * It used to allows overriding of RepositoryApi properties by SearchApi properties
     *
     * @public RepositoryApi
     **/
    public function getRepositoryApi()
    {
        if(!$this->_repositoryApi) {
            if(!empty($this->searchUrl) || !empty($this->autosuggestUrl)) {
                $repositoryApi = clone Yii::app()->repositoryApi;
                if(!empty($this->searchUrl)){
                    $repositoryApi->search = $this->searchUrl;
                }
                if(!empty($this->autosuggestUrl)){
                    $repositoryApi->autosuggest = $this->autosuggestUrl;
                }
                $this->_repositoryApi = $repositoryApi;
            } else {
                $this->_repositoryApi = Yii::app()->repositoryApi;
            }
        }
        return $this->_repositoryApi;
    }
    
    /**
     * Counstructor
     *
     * @return void
     **/
    public function __construct($searchUrl = null, $autosuggestUrl = null)
    {
        $this->searchUrl = $searchUrl;
        $this->autosuggestUrl = $autosuggestUrl;
    }


    /**
     * method for search
     *
     * @param string $phrase             text to find
     * @param bool   $usePersonalisation defines if personalization
     *
     * @return array
     */
    public function search($phrase, $params = array(), $start = null, $count = null)
    {
        if (!$this->_checkPhrase($phrase)) {
            return false;
        }
        $whereToSearch = isset($params['whereToSearch']) ? $params['whereToSearch'] : Yii::app()->getModule('search')->whereToSearch['default'];
        $mimetype = isset($params['mimetype']) ? $params['mimetype'] : Yii::app()->getModule('search')->mimetype['default'];
        $searchType = isset($params['searchType']) ? $params['searchType'] : Yii::app()->getModule('search')->searchType['default'];
        $urlLimiter = isset($params['urlLimiter']) ? $params['urlLimiter'] : Yii::app()->getModule('search')->urlLimiter['default'];
        $enableSearchOperators = isset($params['enableSearchOperators']) ? $params['enableSearchOperators'] : Yii::app()->getModule('search')->searchType['enableSearchOperators'];

        
        $data = array(
            'results' => array(),
            'metaresolvable' => array()
        );
        $data = $this->_preformSearch($phrase, $whereToSearch, $mimetype, $searchType, $urlLimiter, $start, $count, $enableSearchOperators);
        if($data['status'] != 'ok'){
            return false;
        }
        unset($data['status']);
        $meta = $data['10001']['attributes'];
        unset($data['10001']);

        return array(
            'meta' => $meta,
            'results' => $data
        );
    }


    /**
     * Function checks cache if result for given parameters exists and if not - performs API request for it.
     *
     * @param string $phrase        a phrase to search
     * @param string $whereToSearch where to search
     * @param string $mimetype      mimetype of searched content
     * @param string $searchType    search type (AND / OR / NOT / EXACT)
     * @param string $urlLimiter    url of result must contain this
     * @param boolean $enableSearchOperators  precondition to understand OR, AND, NOT as operators
     *
     * @return array|bool
     */
    private function _preformSearch($phrase, $whereToSearch, $mimetype, $searchType, $urlLimiter, $start = null, $count = null, $enableSearchOperators = false)
    {
        // if there are more than 4 words, we search for phrase
        if ($searchType == 'EXACT' || count( explode(' ', $phrase) ) > 4) {
            
            $phrase = '"'.$phrase.'"';

        } else {

            $phrase = preg_replace('/([\ ]+)/', ' ', $phrase);

            /*  gettings quoted phrases */
            $wordGroups = $this->_extractWordGroups($phrase);

            /*  replacing spaces in quoted phrases to avoid splitting */
            if (!empty($wordGroups)) {
                $phrase = str_replace(array_keys($wordGroups), $wordGroups, $phrase);
            }

            /*
             * Feature OR,AND,NOT to understand words OR, AND, NOT as operators
             */
            if (!$enableSearchOperators) {

                $phrase = explode(' ', $phrase);
                $phrase = implode(' '.$searchType.' ', $phrase);

            } else {

                $tmpPhrase = explode(' ', $phrase);
                $phrase = array();
                $keywordFlag = false;

                foreach ($tmpPhrase as $i => $word) {
                    if (in_array($word, array('AND', 'OR', 'NOT'))) {
                        $keywordFlag = true;
                        $phrase[] = $word;
                        continue;
                    }

                    if ($keywordFlag || $i == 0) {
                        $keywordFlag = false;
                        $phrase[] = $word;
                    } else {
                        $phrase[] = $searchType.' '.$word;
                    }
                }

                $phrase = implode(' ', $phrase);
            }
            /*
             * End of feature
             */
            /*  return spaces in their places */
            if (!empty($wordGroups)) {
                $phrase = str_replace($wordGroups, array_keys($wordGroups), $phrase);
            }
        }

        $staticString = Yii::app()->getModule('search')->staticString;
        if (trim($staticString)!='') {
            $phrase .= ' '.$staticString;
        }

        $lang = substr(Yii::app()->language, 0, 2);
        $params = array(
            'type' => 'php',
            'metaresolvable' => 'true',
            'filter' =>  'languagecode:'.$lang,
        );

        if (!empty($this->getRepositoryApi()->additionalRequestParameters['search'])) {
            $additional = $this->getRepositoryApi()->additionalRequestParameters['search'];
            foreach ($additional as $item=>$val) {
                $params[$item] = !empty($params[$item]) ? $params[$item].' AND '.$val : $params[$item];
            }
        }

        if ($whereToSearch != 'all') {
            $params['filter'] .= ' AND '.$whereToSearch.':('.$phrase.')';
        } else {
            $whereToSearchAll = Yii::app()->getModule('search')->whereToSearch['options'];
            if (isset($whereToSearchAll['all'])) {
                unset($whereToSearchAll['all']);
            }

            $aWhereToSearchAll = array();

            foreach ($whereToSearchAll as $key=>$val) {
                $aWhereToSearchAll[] = $key.':('.$phrase.')';
            }

            $params['filter'] .= ' AND ('.implode(' OR ', $aWhereToSearchAll).')';
        }

        if ($mimetype!='all') {
            $params['filter'] .= ' AND mimetype:'.$mimetype;
        }

        if ($urlLimiter!='all') {
            $params['filter'] .= ' AND url:'.$urlLimiter;
        }

        if($start !== null){
            $params['start'] = $start;    
        }

        if($count !== null){
            $params['count'] = $count;    
        }

        $cacheId = md5(serialize($params));

        if (($data = Yii::app()->cache->get($cacheId)) === false) {
            $data = $this->getRepositoryApi()->requestSearch($params);

            $data = unserialize($data);

            /*  store all recieved data in cache */
            Yii::app()->cache->set($cacheId, $data, Yii::app()->getModule('search')->cacheTime);
        }
        return $data;
    }

    /**
     * Checks given phrase for correct syntax
     *
     * @param string $phrase a phrase to search
     *
     * @return bool
     */
    private function _checkPhrase($phrase)
    {
        if (trim($phrase) == '') {
            return false;
        }
        return true;
    }

    private function _extractWordGroups($phrase, $quotes=array('"'))
    {
        if (empty($phrase)) {
            return array();
        }

        $wordGroups = array();
        
        foreach ($quotes as $quote) {
            $reg = preg_quote($quote);
            preg_match_all('/['.$quote.'](.*)['.$quote.']/sU', $phrase, $matches);

            if(!empty($matches[1])) {
                foreach ($matches[1] as $match) {
                    $wordGroups[$match] = str_replace(' ', '_', $match);
                }
            }
        }
        return $wordGroups;
    }
}

?>
